﻿
Import-Module "$EPiServerPackagingScripts\Get-WebConfig.psm1"
Import-Module "$EPiServerPackagingScripts\Set-WebConfig.psm1"
Import-Module "$EPiServerPackagingScripts\Get-ProtectedModulesPath.psm1"
Import-Module "$EPiServerPackagingScripts\Get-RepositoryPath.psm1"
Import-Module "$EPiServerPackagingScripts\Move-EPiServerProtectedModules.psm1"

#Copy config files from addons to nuget packages
Function CopyConfig($srcPath, $desPath)
{
    if (test-path $srcPath)
    {
        # workaround for -exclude. It seems bug get-childitem, I could not use -exclude 
        Get-ChildItem $srcpath -File  | Where-Object {$_.Name -match ".config"} | Where-Object {$_.Name -notmatch "web.config"} | Where-Object {$_.Name -notmatch "module.config"}| Copy-Item -Destination $desPath -Force
    }
}

# Remove dll from modulesbin
Function RemoveFromModulesBin($modulesBinPath ,$packagePath)
{
    if (Test-Path $packagePath)
    {
        Get-ChildItem $packagePath -Recurse "*.dll" |  %{Join-Path -Path $modulesBinPath -ChildPath $_.Name } | Remove-Item -ErrorAction:ignore -Confirm:$false 
    }
}

# Remove old package from module repository
Function RemoveFromModuleRepository($modulesRepositoryPath, $package)
{
	Get-ChildItem -path $modulesRepositoryPath  -Directory  -Filter ($package.id+"*") | Remove-Item -Confirm:$false -Recurse
}


Function SetInstallationMode($projectPath, $installationMode)
{
	# get web.config as xml doc
	$elementsToBeLoaded = (,"episerver.packaging") 
	$config = Get-WebConfig $projectPath $elementsToBeLoaded
	if($config -eq $null -or $config -eq "") 
	{
		return 
	}
	$packagingElement = $config.configuration."episerver.packaging"
	if ($packagingElement -ne $null)
	{
		$packagingElement.SetAttribute("installationMode", $installationMode)
		Set-WebConfig $config $projectPath $elementsToBeLoaded
	}
}

# Convert a package from project to a nuget based package.
# Change the name of addons folder to deleteme and try to install package of the nuget based addons.
# If succeeded then delete the deletme folder otherwise rollback to original folder.
Function ConvertToNugetPackage($projectPath, $package, $packagePath, $modulesRepositoryPath)
{
	#$srcFeed = "http://nuget.episerver.com/feed/packages.svc/"
	$srcFeed = "https://nugetaddons.episerver.com/feed/addons.svc/"

	if (test-path $packagePath)
	{
		$packageDeleteMePath  = $packagePath + ".deleteme"

        #change the name to deletedme
		Rename-Item $packagePath   $packageDeleteMePath  -Force
		
		try
		{
			Install-Package $package.id -Source $srcFeed -FileConflictAction:Overwrite
		    if (Test-Path $packagePath)
            {
                 CopyConfig  $packageDeleteMePath $packagePath
                 RemoveFromModulesBin  (Join-path $projectPath "modulesbin") $packageDeleteMePath
				 RemoveFromModuleRepository $modulesRepositoryPath $package
            }
            Remove-Item $packageDeleteMePath -Confirm:$false -Recurse
			return $true
        }
		catch [Exception]
	    {
		    $errorMessage = $_.Exception.Message		
            if (Test-Path $packagePath)
            {
                Remove-Item $packagePath -Confirm:$false -Recurse
            }

            Rename-Item $packageDeleteMePath $packagePath
			Write-host $errorMessage
        }	
    }
	return $false
}


<#
.SYNOPSIS
	Convert old format of addons to Nuget based format.

.DESCRIPTION	
	This CommandLet converts the old addons inside public/protected folder to nuget based.
	Sets the installation mode in the episerver packaging config to Code.

.PARAMETER projectPath
	The path to project or current project

.EXAMPLE
	Convert Addons for current project
	Convert-EPiAddons
#>
Function Convert-EPiAddons($projectPath)
{	
	# fill up projectPath if not given
	if($projectPath -eq $null -or $projectPath -eq "")
	{
		$name = (Get-Project).FullName 
		$projectPath = Split-Path $name
	}

	# quit if no project path
	if($projectPath -eq $null -or $projectPath -eq "")
	{			
		return 
	}

	# Move old module to the new path
	if (!(ExistsProtectedModulePath $projectPath))
	{
		Move-EPiServerProtectedModules $projectPath
	}

	# get web.config as xml doc
	$elementsToBeLoaded = (,"episerver.framework") # array of elements which we want to expand with web.config
	$config = Get-WebConfig $projectPath $elementsToBeLoaded

	# exit if webConfig is not at root 
	if($config -eq $null -or $config -eq "") 
	{
		return 
	}

	$protectedModulesPath = Get-ProtectedModulesPath $config $projectPath
	$publicModulesPath = Get-PublicModulesPath $config $projectPath
	$modulesRepositoryPath = Get-ModulesRepositoryPath $config $projectPath

    $packageConfigPath = Join-Path $protectedModulesPath "packages.config"
	if (test-path $packageConfigPath)
	{
		$packageConfig = New-Object xml
		$packageConfig.Load($packageConfigPath)

		#for each old addons addons
		foreach ($package in $packageConfig.packages.package)
		{
			# Try to find under protected 
			$packagePath  = Join-Path $protectedModulesPath $package.id
			$res = (ConvertToNugetPackage $projectPath $package $packagePath $modulesRepositoryPath)
			if ($res -eq $true)
			{	
				[Void]$packageConfig.packages.RemoveChild($package)
				continue
			}

			# Try to find under appdata/modules
			$packagePath  = Join-Path $publicModulesPath $package.id
			$res = (ConvertToNugetPackage  $projectPath $package $packagePath $modulesRepositoryPath)
			if ($res -eq $true)
			{	
				[Void]$packageConfig.packages.RemoveChild($package)
				continue
			}

			# Try to find under projectpath modules repository
			$packagePath  = Join-Path $projectPath "modules"
			$packagePath  = Join-Path $packagePath  $package.id
			$res = (ConvertToNugetPackage $projectPath $package $packagePath $modulesRepositoryPath)
			if ($res -eq $true)
			{	
				[Void]$packageConfig.packages.RemoveChild($package)
				continue
			}

		}
		[Void]$packageConfig.Save($packageConfigPath)
	}
	
	SetInstallationMode $projectPath "Code"
}

Export-ModuleMember -Function  Convert-EPiAddons
# SIG # Begin signature block
# MIIZBwYJKoZIhvcNAQcCoIIY+DCCGPQCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU+O7621X7Avb2o3ODlQOpkFvy
# uwKgghP/MIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggVUMIIEPKADAgECAhBqBz1Yk9Ce+JomHWkTBhgAMA0GCSqGSIb3DQEBBQUAMIG0
# MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsT
# FlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBh
# dCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYDVQQDEyVW
# ZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmluZyAyMDEwIENBMB4XDTEzMDIwNTAw
# MDAwMFoXDTE2MDQwNTIzNTk1OVowgZcxCzAJBgNVBAYTAlNFMQowCAYDVQQIEwEt
# MQ4wDAYDVQQHEwVLSVNUQTEVMBMGA1UEChQMRVBpU2VydmVyIEFCMT4wPAYDVQQL
# EzVEaWdpdGFsIElEIENsYXNzIDMgLSBNaWNyb3NvZnQgU29mdHdhcmUgVmFsaWRh
# dGlvbiB2MjEVMBMGA1UEAxQMRVBpU2VydmVyIEFCMIIBIjANBgkqhkiG9w0BAQEF
# AAOCAQ8AMIIBCgKCAQEAo6coNqVVn2Rk4HBEl0kc/HO+PttBuDrEx/9fKLONe3yT
# SFWk6dg7/Lv1l+uSTwt4GbWkk3HU4tRRd2gPJ3AK14AysycQRE9T0H5mhcJntXnz
# 6i6rOOQjEqmjipcu1iO1BPl8OSEK3h37kjjhtPCei2KpViH2icmVfVgtevF988qh
# n7V/B66QtQGjl44gBAI3JBgUDUhCCFO+d9+tY6gYx9SR9+OwYWNusRpEG4wHlzpo
# mK4xIcrk6CBfktyEjDRs7ZCNOdL3mWvPKVeZjj3+f+XPrARmEOkBsCCDuRPG2bRK
# /3gLrAfVP5L73EHHNgLqS2uzPppChulRvIKjnUyOVwIDAQABo4IBezCCAXcwCQYD
# VR0TBAIwADAOBgNVHQ8BAf8EBAMCB4AwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDov
# L2NzYzMtMjAxMC1jcmwudmVyaXNpZ24uY29tL0NTQzMtMjAxMC5jcmwwRAYDVR0g
# BD0wOzA5BgtghkgBhvhFAQcXAzAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy52
# ZXJpc2lnbi5jb20vcnBhMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHEGCCsGAQUFBwEB
# BGUwYzAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNpZ24uY29tMDsGCCsG
# AQUFBzAChi9odHRwOi8vY3NjMy0yMDEwLWFpYS52ZXJpc2lnbi5jb20vQ1NDMy0y
# MDEwLmNlcjAfBgNVHSMEGDAWgBTPmanqeyb0S8mOj9fwBSbv49KnnTARBglghkgB
# hvhCAQEEBAMCBBAwFgYKKwYBBAGCNwIBGwQIMAYBAQABAf8wDQYJKoZIhvcNAQEF
# BQADggEBAIk7DJSHwVYLgVDJzo1GSsMElW0XhAkl167CGHP0q18xgRCEZsb1M93u
# Z6uSnJWbtnGrxHSjbOxvWPUQSChMq7h+aZdw/emdFpZ5g3tbKcTZN/1l8pREvPG7
# vO/UUmXSG20xezxcuzM2bRgIYxmFIHNn6XXVORkWVGujm/zo/dYVDPjH1udFZ5nj
# IenD/YeO2ZjvnZssAoyTZuDhpf3qUtEff2Kc+PXVYoMsk8Q4TO74ps6DbpqddLDg
# k73Xbyr++tvmhZIL8XSzB9j1thEijIwYFn6k2TMls4pVQ8s/37oJcvwZ/KPICLUY
# +A8+Kx6iywx7QN1mfwEekzsKiYA7LHswggYKMIIE8qADAgECAhBSAOWqJVb8Gobt
# lsnUSzPHMA0GCSqGSIb3DQEBBQUAMIHKMQswCQYDVQQGEwJVUzEXMBUGA1UEChMO
# VmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsx
# OjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMuIC0gRm9yIGF1dGhvcml6
# ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENsYXNzIDMgUHVibGljIFBy
# aW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHNTAeFw0xMDAyMDgwMDAw
# MDBaFw0yMDAyMDcyMzU5NTlaMIG0MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVy
# aVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5
# BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20v
# cnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmlu
# ZyAyMDEwIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA9SNLXqXX
# irsy6dRX9+/kxyZ+rRmY/qidfZT2NmsQ13WBMH8EaH/LK3UezR0IjN9plKc3o5x7
# gOCZ4e43TV/OOxTuhtTQ9Sc1vCULOKeMY50Xowilq7D7zWpigkzVIdob2fHjhDuK
# Kk+FW5ABT8mndhB/JwN8vq5+fcHd+QW8G0icaefApDw8QQA+35blxeSUcdZVAccA
# JkpAPLWhJqkMp22AjpAle8+/PxzrL5b65Yd3xrVWsno7VDBTG99iNP8e0fRakyiF
# 5UwXTn5b/aSTmX/fze+kde/vFfZH5/gZctguNBqmtKdMfr27Tww9V/Ew1qY2jtaA
# dtcZLqXNfjQtiQIDAQABo4IB/jCCAfowEgYDVR0TAQH/BAgwBgEB/wIBADBwBgNV
# HSAEaTBnMGUGC2CGSAGG+EUBBxcDMFYwKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3
# LnZlcmlzaWduLmNvbS9jcHMwKgYIKwYBBQUHAgIwHhocaHR0cHM6Ly93d3cudmVy
# aXNpZ24uY29tL3JwYTAOBgNVHQ8BAf8EBAMCAQYwbQYIKwYBBQUHAQwEYTBfoV2g
# WzBZMFcwVRYJaW1hZ2UvZ2lmMCEwHzAHBgUrDgMCGgQUj+XTGoasjY5rw8+AatRI
# GCx7GS4wJRYjaHR0cDovL2xvZ28udmVyaXNpZ24uY29tL3ZzbG9nby5naWYwNAYD
# VR0fBC0wKzApoCegJYYjaHR0cDovL2NybC52ZXJpc2lnbi5jb20vcGNhMy1nNS5j
# cmwwNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC52ZXJp
# c2lnbi5jb20wHQYDVR0lBBYwFAYIKwYBBQUHAwIGCCsGAQUFBwMDMCgGA1UdEQQh
# MB+kHTAbMRkwFwYDVQQDExBWZXJpU2lnbk1QS0ktMi04MB0GA1UdDgQWBBTPmanq
# eyb0S8mOj9fwBSbv49KnnTAfBgNVHSMEGDAWgBR/02Wnwt3su/AwCfNDOfoCrzMx
# MzANBgkqhkiG9w0BAQUFAAOCAQEAViLmNKTEYctIuQGtVqhkD9mMkcS7zAzlrXqg
# In/fRzhKLWzRf3EafOxwqbHwT+QPDFP6FV7+dJhJJIWBJhyRFEewTGOMu6E01MZF
# 6A2FJnMD0KmMZG3ccZLmRQVgFVlROfxYFGv+1KTteWsIDEFy5zciBgm+I+k/RJoe
# 6WGdzLGQXPw90o2sQj1lNtS0PUAoj5sQzyMmzEsgy5AfXYxMNMo82OU31m+lIL00
# 6ybZrg3nxZr3obQhkTNvhuhYuyV8dA5Y/nUbYz/OMXybjxuWnsVTdoRbnK2R+qzt
# k7pdyCFTwoJTY68SDVCHERs9VFKWiiycPZIaCJoFLseTpUiR0zGCBHIwggRuAgEB
# MIHJMIG0MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAd
# BgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9m
# IHVzZSBhdCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYD
# VQQDEyVWZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmluZyAyMDEwIENBAhBqBz1Y
# k9Ce+JomHWkTBhgAMAkGBSsOAwIaBQCgcDAQBgorBgEEAYI3AgEMMQIwADAZBgkq
# hkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGC
# NwIBFTAjBgkqhkiG9w0BCQQxFgQUj1ACKj4Oor2ccSAdLSbZuF0dZiMwDQYJKoZI
# hvcNAQEBBQAEggEAP2CcqO73t5IIZXhhefRV0HXoYPHu5lrKMHffiB+TRyy5lklf
# hdrWfEpiR66M8WZiRMj936QfKUGKxImsVrvVCOj1OVjKF9gb4QAOS703myOiaaQr
# paTiYfIbxF2ZDh7vcJ0EfIuYrqTar02JNz5cXrg6036k0d6q55p8iZd3X8ThYiPg
# 1sxtO1zuswa7/udjk5F64IM4RjhMMQT7fudweXav7NKaqR7YWa1RKosulO37Y2zy
# xW+KRN+7YgWahpBtX3dB/QsHWQyibfUYLzzz6SCBjCFrXfIPeredXRLWwJIt/FSR
# NG1tH5903ONFO2FKYT5cJj1Jgw4s170eTYrsE6GCAgswggIHBgkqhkiG9w0BCQYx
# ggH4MIIB9AIBATByMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBD
# b3Jwb3JhdGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2
# aWNlcyBDQSAtIEcyAhAOz/Q4yP6/NW4E2GqYGxpQMAkGBSsOAwIaBQCgXTAYBgkq
# hkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0xNTAzMTMxMDAx
# MDNaMCMGCSqGSIb3DQEJBDEWBBQ49lMfa77T04l/B1WZplGo35QgyTANBgkqhkiG
# 9w0BAQEFAASCAQCJ88FscaQlEyxaXMxLLHDo26ZEUT8Lcv9nxEuSZQQmZpTYB60G
# m9CWfVSnjKfrLLGwlA7XkNjk+FXtCYsnpvSdLLMjC7Ht3+US5gcjn6kRWeDRuS82
# VKgrhMV5aFxnbWJIKZSzZ9r5OwOSaw5osevLhILu52YFZBKEdMd68U7JrAEVAknI
# M/pgMjj+dNsvD4AW+B98QBzcSH9jP9daF3pMfDPkrls/Ek6311Jghiitf0/Y3Lch
# BE8pC+1JgT5mr7eGrM7u+IsWzbFmAG44lj0CpAacalpIpAyLsOxefm0Zyv32sr8y
# VtSGw+grUlNcZEJsaeyi5L9cVC/rIgIE/d0A
# SIG # End signature block
